/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcClientConfigImpl;
import org.xwiki.xmlrpc.model.Attachment;
import org.xwiki.xmlrpc.model.Comment;
import org.xwiki.xmlrpc.model.Page;
import org.xwiki.xmlrpc.model.PageHistorySummary;
import org.xwiki.xmlrpc.model.PageSummary;
import org.xwiki.xmlrpc.model.ServerInfo;
import org.xwiki.xmlrpc.model.Space;
import org.xwiki.xmlrpc.model.SpaceSummary;
import org.xwiki.xmlrpc.model.XWikiClass;
import org.xwiki.xmlrpc.model.XWikiClassSummary;
import org.xwiki.xmlrpc.model.XWikiObject;
import org.xwiki.xmlrpc.model.XWikiObjectSummary;

/**
 * This class implements the entry point for using the XWiki XMLRPC API. Methods compatible with
 * Confluence are tagged with the category 'ConfluenceAPI'
 * 
 * @author fmancinelli
 */
public class XWikiXmlRpcClient
{
    private XmlRpcClient xmlRpcClient;

    private String token;

    private String rpcHandler;

    /**
     * Constructor.
     * 
     * @param endpoint The endpoint for the XMLRPC servlet.
     * @throws MalformedURLException
     */
    public XWikiXmlRpcClient(String endpoint) throws MalformedURLException
    {
        this(endpoint, "xwiki1");
    }

    /**
     * Constructor.
     * 
     * @param endpoint The endpoint for the XMLRPC servlet.
     * @param rpcHandler The id of the XMLRPC handler.
     * @throws MalformedURLException
     */
    public XWikiXmlRpcClient(String endpoint, String rpcHandler) throws MalformedURLException
    {
        XmlRpcClientConfigImpl clientConfig = new XmlRpcClientConfigImpl();
        clientConfig.setServerURL(new URL(endpoint));

        xmlRpcClient = new XmlRpcClient();
        xmlRpcClient.setConfig(clientConfig);

        this.rpcHandler = rpcHandler;

        token = "";
    }

    /***********************************************************************************************
     * Authentication ******************************************************************************
     **********************************************************************************************/

    /**
     * Login.
     * 
     * @param username
     * @param password
     * @throws XmlRpcException If an invalid username/password was specified or communication
     *             problem.
     */
    public void login(String username, String password) throws XmlRpcException
    {
        token = (String) invokeRpc("login", username, password);
    }

    /**
     * Logout.
     * 
     * @return True if logout was succesfull.
     * @throws XmlRpcException
     */
    public boolean logout() throws XmlRpcException
    {
        Boolean value = (Boolean) invokeRpc("logout", token);
        token = "";

        return value.booleanValue();
    }

    /***********************************************************************************************
     * Information *********************************************************************************
     **********************************************************************************************/

    /**
     * Get server info.
     * 
     * @return Server information.
     * @throws XmlRpcException
     */
    public ServerInfo getServerInfo() throws XmlRpcException
    {
        Object object = invokeRpc("getServerInfo", token);
        return new ServerInfo((Map<String, Object>) object);
    }

    /***********************************************************************************************
     * Spaces **************************************************************************************
     **********************************************************************************************/

    /**
     * Get spaces.
     * 
     * @return A list of SpaceSummaries for each space defined in the remote XWiki.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public List<SpaceSummary> getSpaces() throws XmlRpcException
    {
        Object[] objects = (Object[]) invokeRpc("getSpaces", token);
        List<SpaceSummary> result = new ArrayList<SpaceSummary>();
        for (Object object : objects) {
            Map<String, Object> spaceSummaryMap = (Map<String, Object>) object;
            SpaceSummary spaceSummary = new SpaceSummary(spaceSummaryMap);
            result.add(spaceSummary);
        }

        return result;
    }

    /**
     * Get information about a given space.
     * 
     * @param spaceName
     * @return A Space object containing extended space information.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Space getSpace(String spaceName) throws XmlRpcException
    {
        Object object = invokeRpc("getSpace", token, spaceName);
        return new Space((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getSpace(String)
     */
    public Space getSpace(SpaceSummary spaceSummary) throws XmlRpcException
    {
        return getSpace(spaceSummary.getKey());
    }

    /**
     * Add a new space.
     * 
     * @param space The Space object containing information about the new space.
     * @return A Space object with up-to-date information about the newly created space.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Space addSpace(Space space) throws XmlRpcException
    {
        Object object = invokeRpc("addSpace", token, space.toMap());
        return new Space((Map<String, Object>) object);
    }

    /**
     * Remove space and all of its pages.
     * 
     * @param spaceName
     * @return True if the space was successfully deleted.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Boolean removeSpace(String spaceKey) throws XmlRpcException
    {
        return (Boolean) invokeRpc("removeSpace", token, spaceKey);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#removeSpace(String)
     */
    public Boolean removeSpace(SpaceSummary spaceSummary) throws XmlRpcException
    {
        return removeSpace(spaceSummary.getKey());
    }

    /***********************************************************************************************
     * Pages ***************************************************************************************
     **********************************************************************************************/

    /**
     * Get pages in space.
     * 
     * @param spaceName
     * @return A list of SpaceSummaries for each page in the given space.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public List<PageSummary> getPages(String spaceName) throws XmlRpcException
    {
        Object[] objects = (Object[]) invokeRpc("getPages", token, spaceName);
        List<PageSummary> result = new ArrayList<PageSummary>();
        for (Object object : objects) {
            Map<String, Object> pageSummaryMap = (Map<String, Object>) object;
            PageSummary pageSummary = new PageSummary(pageSummaryMap);
            result.add(pageSummary);
        }

        return result;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getPages(String)
     */
    public List<PageSummary> getPages(SpaceSummary space) throws XmlRpcException
    {
        return getPages(space.getKey());
    }

    /**
     * Get full page information and content.
     * 
     * @param pageId
     * @return A Page object containing all the information.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Page getPage(String pageId) throws XmlRpcException
    {
        Object object = invokeRpc("getPage", token, pageId);
        return new Page((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getPage(String)
     */
    public Page getPage(PageSummary pageSummary) throws XmlRpcException
    {
        return getPage(pageSummary.getId());
    }

    /**
     * Get full page information and content in a given language.
     * 
     * @param pageId
     * @param language
     * @return A Page object containing all the information.
     * @throws XmlRpcException
     */
    public Page getPage(String pageId, String language) throws XmlRpcException
    {
        Object object = invokeRpc("getPage", token, pageId, language);
        return new Page((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getPage(String, String)
     */
    public Page getPage(PageSummary pageSummary, String language) throws XmlRpcException
    {
        return getPage(pageSummary.getId(), language);
    }

    /**
     * Get full page information and content at a given version.
     * 
     * @param pageId
     * @param version
     * @return A Page object containing all the information.
     * @throws XmlRpcException
     */
    public Page getPage(String pageId, Integer version) throws XmlRpcException
    {
        Object object = invokeRpc("getPage", token, pageId, version);
        return new Page((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getPage(String, Integer)
     */
    public Page getPage(PageSummary pageSummary, Integer version) throws XmlRpcException
    {
        return getPage(pageSummary.getId(), version);
    }

    /**
     * Get full page information and content at a given version.
     * 
     * @param pageId
     * @param version
     * @return A Page object containing all the information.
     * @throws XmlRpcException
     */
    private Page getPage(String pageId, Integer version, Integer minorVersion)
        throws XmlRpcException
    {
        Object object = invokeRpc("getPage", token, pageId, version, minorVersion);
        return new Page((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getPage(String, Integer, Integer)
     */
    public Page getPage(PageHistorySummary pageHistorySummary) throws XmlRpcException
    {
        return getPage(pageHistorySummary.getId(), pageHistorySummary.getVersion(),
            pageHistorySummary.getMinorVersion());
    }

    /**
     * Get the history of a page.
     * 
     * @param pageId
     * @return A list of PageHistorySummary for each revision of the page.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public List<PageHistorySummary> getPageHistory(String pageId) throws XmlRpcException
    {
        Object[] objects = (Object[]) invokeRpc("getPageHistory", token, pageId);
        List<PageHistorySummary> result = new ArrayList<PageHistorySummary>();
        for (Object object : objects) {
            Map<String, Object> pageHistorySummaryMap = (Map<String, Object>) object;
            PageHistorySummary pageHistorySummary = new PageHistorySummary(pageHistorySummaryMap);
            result.add(pageHistorySummary);
        }

        return result;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getPageHistory(String)
     */
    public List<PageHistorySummary> getPageHistory(PageSummary pageSummary)
        throws XmlRpcException
    {
        return getPageHistory(pageSummary.getId());
    }

    /**
     * Store a page.
     * 
     * @param page
     * @return The Page object with the up-to-date information about the newly stored page.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Page storePage(Page page) throws XmlRpcException
    {
        Object object = invokeRpc("storePage", token, page.toMap());
        return new Page((Map<String, Object>) object);
    }

    /**
     * Remove a page.
     * 
     * @param pageId
     * @return True if the page has been successfully removed.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Boolean removePage(String pageId) throws XmlRpcException
    {
        Object object = invokeRpc("removePage", token, pageId);
        return (Boolean) object;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#removePage(String)
     */
    public Boolean removePage(PageSummary page) throws XmlRpcException
    {
        return removePage(page.getId());
    }
    
    /**
     * Render a page or content in HTML.
     * 
     * @param space Ignored
     * @param pageId The page id in the form of Space.Page
     * @param content The content to be rendered. If content == "" then the page content is rendered.
     * @return The rendered content.
     * @throws XWikiException
     * @throws XmlRpcException XmlRpcException If the page does not exist or the user has not the right to access
     *             it.
     * @category ConfluenceAPI
     */
    public String renderContent(String space, String pageId, String content) throws XmlRpcException {
        return (String)invokeRpc("renderContent", token, space, pageId, content);
    }

    /**
     * Get page comments.
     * 
     * @param pageId
     * @return A list of Comment objects for each comment associated to the page.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public List<Comment> getComments(String pageId) throws XmlRpcException
    {
        Object[] objects = (Object[]) invokeRpc("getComments", token, pageId);
        List<Comment> result = new ArrayList<Comment>();
        for (Object object : objects) {
            Map<String, Object> commentMap = (Map<String, Object>) object;
            Comment comment = new Comment(commentMap);
            result.add(comment);
        }

        return result;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getComments(String)
     */
    public List<Comment> getComments(PageSummary pageSummary) throws XmlRpcException
    {
        return getComments(pageSummary.getId());
    }

    /**
     * Add a new comment.
     * 
     * @param pageId
     * @param comment A Comment object containing the comment to be added.
     * @return A Comment object containing the information about the newly created comment.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Comment addComment(String pageId, Comment comment) throws XmlRpcException
    {
        Object object = invokeRpc("addComment", token, pageId, comment.toMap());
        return new Comment((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#addComment(String, Comment)
     */
    public Comment addComment(PageSummary pageSummary, Comment comment) throws XmlRpcException
    {
        return addComment(pageSummary.getId(), comment);
    }

    /**
     * Remove a comment.
     * 
     * @param pageId
     * @param commentId
     * @return True if the comment has been successfully removed.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Boolean removeComment(String pageId, String commentId) throws XmlRpcException
    {
        Object object = invokeRpc("removeComment", token, pageId, commentId);
        return (Boolean) object;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#removeComment(String, String)
     */
    public Boolean removeComment(PageSummary pageSummary, Comment comment) throws XmlRpcException
    {
        return removeComment(pageSummary.getId(), comment.getId());
    }

    /**
     * Get attachments.
     * 
     * @param pageId
     * @return A list of Attachment objects describing the attachments associated to the page.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public List<Attachment> getAttachments(String pageId) throws XmlRpcException
    {
        Object[] objects = (Object[]) invokeRpc("getAttachments", token, pageId);
        List<Attachment> result = new ArrayList<Attachment>();
        for (Object object : objects) {
            Map<String, Object> attachmentMap = (Map<String, Object>) object;
            Attachment attachment = new Attachment(attachmentMap);
            result.add(attachment);
        }

        return result;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getAttachments(String)
     */
    public List<Attachment> getAttachments(PageSummary pageSummary) throws XmlRpcException
    {
        return getAttachments(pageSummary.getId());
    }

    /**
     * Get the binary data associated to the attachment.
     * 
     * @param pageId
     * @param fileName
     * @param versionNumber Ignored.
     * @return The actual attachment data.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public byte[] getAttachmentData(String pageId, String fileName, String versionNumber)
        throws XmlRpcException
    {
        return (byte[]) invokeRpc("getAttachmentData", token, pageId, fileName, versionNumber);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getAttachmentData(String, String, String)
     */
    public byte[] getAttachmentData(Attachment attachment) throws XmlRpcException
    {
        return getAttachmentData(attachment.getPageId(), attachment.getFileName(), "");
    }

    /**
     * Add a new attachment.
     * 
     * @param contentId Ignored. (It is here because the Confluence API signature declares it)
     * @param attachment The Attachment object describing the attachment.
     * @param attachmentData The actual attachment data.
     * @return An Attachment object describing the newly added attachment.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Attachment addAttachment(Integer contentId, Attachment attachment,
        byte[] attachmentData) throws XmlRpcException
    {
        Object object =
            invokeRpc("addAttachment", token, contentId, attachment.toMap(), attachmentData);
        return (Attachment) new Attachment((Map<String, Object>) object);
    }

    /**
     * Remove attachment.
     * 
     * @param pageId
     * @param fileName
     * @return True if the attachment has been successfully removed.
     * @throws XmlRpcException
     * @category ConfluenceAPI
     */
    public Boolean removeAttachment(String pageId, String fileName) throws XmlRpcException
    {
        return (Boolean) invokeRpc("removeAttachment", token, pageId, fileName);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#removeAttachment(String, String)
     */
    public Boolean removeAttachment(Attachment attachment) throws XmlRpcException
    {
        return removeAttachment(attachment.getPageId(), attachment.getFileName());
    }

    /***********************************************************************************************
     * Classes *************************************************************************************
     **********************************************************************************************/

    /**
     * Get classes.
     * 
     * @return A list of XWikiClassSummaries for each class defined in the XWiki instance.
     * @throws XmlRpcException
     */
    public List<XWikiClassSummary> getClasses() throws XmlRpcException
    {
        Object[] objects = (Object[]) invokeRpc("getClasses", token);
        List<XWikiClassSummary> result = new ArrayList<XWikiClassSummary>();
        for (Object object : objects) {
            Map<String, Object> xwikiClassSummaryMap = (Map<String, Object>) object;
            XWikiClassSummary xwikiClassSummary = new XWikiClassSummary(xwikiClassSummaryMap);
            result.add(xwikiClassSummary);
        }

        return result;
    }

    /**
     * Get extended information about a class.
     * 
     * @param className
     * @return A XWikiClass object with the extended information.
     * @throws XmlRpcException
     */
    public XWikiClass getClass(String className) throws XmlRpcException
    {
        Object object = invokeRpc("getClass", token, className);
        return new XWikiClass((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getClass(String)
     */
    public XWikiClass getClass(XWikiClassSummary classSummary) throws XmlRpcException
    {
        return getClass(classSummary.getId());
    }

    /**
     * Get XWiki objects associated with a page.
     * 
     * @param pageId
     * @return A list containing an XWikiObject summary for every object found in the page.
     * @throws XmlRpcException
     */
    public List<XWikiObjectSummary> getObjects(String pageId) throws XmlRpcException
    {
        Object[] objects = (Object[]) invokeRpc("getObjects", token, pageId);
        List<XWikiObjectSummary> result = new ArrayList<XWikiObjectSummary>();
        for (Object object : objects) {
            Map<String, Object> xwikiObjectSummaryMap = (Map<String, Object>) object;
            XWikiObjectSummary xwikiObjectSummary = new XWikiObjectSummary(xwikiObjectSummaryMap);
            result.add(xwikiObjectSummary);
        }

        return result;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getObjects(PageSummary)
     */
    public List<XWikiObjectSummary> getObjects(PageSummary pageSummary) throws XmlRpcException
    {
        return getObjects(pageSummary.getId());
    }

    /**
     * Get XWiki object.
     * 
     * @param pageId
     * @param className
     * @param id The XWiki object id.
     * @return An XWiki object with all the association property -> values.
     * @throws XmlRpcException
     */
    public XWikiObject getObject(String pageId, String className, Integer id)
        throws XmlRpcException
    {
        Object object = invokeRpc("getObject", token, pageId, className, id);
        return new XWikiObject((Map<String, Object>) object);
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#getObject(String, String, Integer)
     */
    public XWikiObject getObject(XWikiObjectSummary xwikiObjectSummary) throws XmlRpcException
    {
        return getObject(xwikiObjectSummary.getPageId(), xwikiObjectSummary.getClassName(),
            xwikiObjectSummary.getId());
    }

    /**
     * Store an XWiki object.
     * 
     * @param xwikiObject
     * @return An XWikiObject with the up-to-date information about the stored object.
     * @throws XmlRpcException
     */
    public XWikiObject storeObject(XWikiObject xwikiObject) throws XmlRpcException
    {
        Object object = invokeRpc("storeObject", token, xwikiObject.toMap());
        return new XWikiObject((Map<String, Object>) object);
    }

    /**
     * Remove an XWiki Object.
     * 
     * @param pageId
     * @param className
     * @param id
     * @return True if the XWiki objec has been successfully removed.
     * @throws XmlRpcException
     */
    public Boolean removeObject(String pageId, String className, Integer id)
        throws XmlRpcException
    {
        Object object = invokeRpc("removeObject", token, pageId, className, id);
        return (Boolean) object;
    }

    /**
     * Convenience method.
     * 
     * @see XWikiXmlRpcClient#removeObject(String, String, Integer)
     */
    public Boolean removeObject(XWikiObjectSummary objectSummary) throws XmlRpcException
    {
        return removeObject(objectSummary.getPageId(), objectSummary.getClassName(),
            objectSummary.getId());
    }

    /**
     * Utility method for invoking remote RPC methods.
     * 
     * @param methodName
     * @param args
     * @return
     * @throws XmlRpcException
     */
    private synchronized Object invokeRpc(String methodName, Object... args)
        throws XmlRpcException
    {
        return xmlRpcClient.execute(String.format("%s.%s", rpcHandler, methodName), args);
    }
}
