/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc.model;

import java.util.Date;
import java.util.Map;

/**
 * A class representing attachments. Modeled after
 * {@link http://confluence.atlassian.com/display/DOC/Remote+API+Specification#RemoteAPISpecification-Attachment}
 * 
 * @author fmancinelli
 * @category ConfluenceAPI
 */
public class Attachment extends XmlRpcObject
{
    @XmlRpcObjectField
    protected String id;

    @XmlRpcObjectField
    protected String pageId;

    @XmlRpcObjectField
    protected String title;

    @XmlRpcObjectField
    protected String fileName;

    @XmlRpcObjectField
    protected String fileSize;

    @XmlRpcObjectField
    protected String contentType;

    @XmlRpcObjectField
    protected Date created;

    @XmlRpcObjectField
    protected String creator;

    @XmlRpcObjectField
    protected String url;

    @XmlRpcObjectField
    protected String comment;

    /**
     * Constructor for creating new attachments.
     * 
     * @param pageId
     * @param fileName
     */
    public Attachment(String pageId, String fileName)
    {
        this("", pageId, "", fileName, "", "", new Date(), "", "", "");
        checkFields();
    }

    /**
     * Default constructor for creating Attachment objects. Client should not need to use this.
     * 
     * @param id
     * @param pageId
     * @param title
     * @param fileName
     * @param fileSize
     * @param contentType
     * @param created
     * @param creator
     * @param url
     * @param comment
     */
    public Attachment(String id, String pageId, String title, String fileName, String fileSize,
        String contentType, Date created, String creator, String url, String comment)
    {
        super();

        this.id = id;
        this.pageId = pageId;
        this.title = title;
        this.fileName = fileName;
        this.fileSize = fileSize;
        this.contentType = contentType;
        this.created = created;
        this.creator = creator;
        this.url = url;
        this.comment = comment;

        checkFields();
    }

    /**
     * @param map A map containing the associations between XmlRpcObject field names and their
     *            corresponding values.
     * @see XmlRpcObject#XmlRpcObject(Map)
     */
    public Attachment(Map<String, Object> map)
    {
        super(map);
        checkFields();
    }

    /**
     * Initialize and check that all the fields have correct values.
     */
    private void checkFields()
    {
        if (id == null) {
            throw new IllegalArgumentException("id must not be null");
        }

        if (pageId == null) {
            throw new IllegalArgumentException("pageId must not be null");
        }

        if (title == null) {
            title = "No title";
        }

        if (fileName == null) {
            throw new IllegalArgumentException("fileName must not be null");
        }

        if (fileSize == null) {
            throw new IllegalArgumentException("fileSize must not be null");
        }

        if (contentType == null) {
            throw new IllegalArgumentException("contentType must not be null");
        }

        if (created == null) {
            throw new IllegalArgumentException("created must not be null");
        }

        if (creator == null) {
            throw new IllegalArgumentException("creator must not be null");
        }

        if (url == null) {
            throw new IllegalArgumentException("url must not be null");
        }

        if (comment == null) {
            comment = "";
        }
    }

    public String getId()
    {
        return id;
    }

    public String getPageId()
    {
        return pageId;
    }

    public String getTitle()
    {
        return title;
    }

    public String getFileName()
    {
        return fileName;
    }

    public Integer getFileSize()
    {
        return new Integer(fileSize);
    }

    public String getContentType()
    {
        return contentType;
    }

    public Date getCreated()
    {
        return created;
    }

    public String getCreator()
    {
        return creator;
    }

    public String getUrl()
    {
        return url;
    }

    public String getComment()
    {
        return comment;
    }
}
