/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc.model;

import java.util.Date;
import java.util.Map;

/**
 * A class representing comments. Modeled after
 * {@link http://confluence.atlassian.com/display/DOC/Remote+API+Specification#RemoteAPISpecification-Comment}
 * 
 * @author fmancinelli
 * @category ConfluenceAPI
 */
public class Comment extends XmlRpcObject
{
    @XmlRpcObjectField
    protected String id;

    @XmlRpcObjectField
    protected String pageId;

    @XmlRpcObjectField
    protected String title;

    @XmlRpcObjectField
    protected String content;

    @XmlRpcObjectField
    protected String url;

    @XmlRpcObjectField
    protected Date created;

    @XmlRpcObjectField
    protected String creator;

    /**
     * Constructor for creating new comments.
     * 
     * @param pageId
     * @param content
     */
    public Comment(String pageId, String content)
    {
        this("NEW", pageId, null, content, null, null, null);
        checkFields();
    }

    /**
     * Default constructor for creating Comment objects. Client should not need to use this.
     * 
     * @param id
     * @param pageId
     * @param title
     * @param content
     * @param url
     * @param created
     * @param creator
     */
    public Comment(String id, String pageId, String title, String content, String url,
        Date created, String creator)
    {
        super();

        this.id = id;
        this.pageId = pageId;
        this.title = title;
        this.content = content;
        this.url = url;
        this.created = created;
        this.creator = creator;

        checkFields();
    }

    /**
     * @param map A map containing the associations between XmlRpcObject field names and their
     *            corresponding values.
     * @see XmlRpcObject#XmlRpcObject(Map)
     */
    public Comment(Map<String, Object> map)
    {
        super(map);
        checkFields();
    }

    /**
     * Initialize and check that all the fields have correct values.
     */
    private void checkFields()
    {
        if (id == null) {
            throw new IllegalArgumentException("Comment id must not be null");
        }

        if (pageId == null) {
            throw new IllegalArgumentException("Page id must not be null");
        }

        if (title == null) {
            title = "Comment";
        }

        if (content == null) {
            content = "";
        }

        if (url == null) {
            url = "";
        }

        if (created == null) {
            created = new Date();
        }

        if (creator == null) {
            creator = "";
        }
    }

    public String getTitle()
    {
        return title;
    }

    public void setTitle(String title)
    {
        this.title = title;
    }

    public String getContent()
    {
        return content;
    }

    public void setContent(String content)
    {
        this.content = content;
    }

    public String getId()
    {
        return id;
    }

    public String getPageId()
    {
        return pageId;
    }

    public String getUrl()
    {
        return url;
    }

    public Date getCreated()
    {
        return created;
    }

    public String getCreator()
    {
        return creator;
    }
}
