/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc.model;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * A class representing pages. Modeled after
 * {@link http://confluence.atlassian.com/display/DOC/Remote+API+Specification#RemoteAPISpecification-Page}
 * The category XWikiSpecific is used to mark fields that have been added with respect to the
 * previous specification.
 * 
 * @author fmancinelli
 * @category ConfluenceAPI
 */
public class Page extends PageSummary
{
    /**
     * A constant representing the default language id for a page.
     * 
     * @category XWikiSpecific
     */
    public static final String DEFAULT_LANGUAGE = "";

    @XmlRpcObjectField
    protected Integer version;

    @XmlRpcObjectField
    protected Integer minorVersion;

    @XmlRpcObjectField
    protected String content;

    @XmlRpcObjectField
    protected Date created;

    @XmlRpcObjectField
    protected String creator;

    @XmlRpcObjectField
    protected Date modified;

    @XmlRpcObjectField
    protected String modifier;

    @XmlRpcObjectField
    protected Boolean homePage;

    @XmlRpcObjectField
    protected String contentStatus;

    @XmlRpcObjectField
    protected Boolean current;

    /**
     * @category XWikiSpecific
     */
    @XmlRpcObjectField
    protected String language;

    /**
     * Constructor for creating new pages in the default language.
     * 
     * @param id
     * @param space
     * @param title
     * @param content
     */
    public Page(String id, String space, String title, String content)
    {
        this(id, space, "", title, "", 0, null, 0, 0, content, new Date(), "", new Date(), "",
            false, "", true, DEFAULT_LANGUAGE);
        checkFields();
    }

    /**
     * Constructor for creating new pages in the specified language.
     * 
     * @param id
     * @param space
     * @param title
     * @param content
     * @param language
     */
    public Page(String id, String space, String title, String content, String language)
    {
        this(id, space, "", title, "", 0, null, 0, 0, content, new Date(), "", new Date(), "",
            false, "", true, language);
        checkFields();
    }

    /**
     * Default constructor for creating Page objects. Client should not need to use this.
     * 
     * @param id
     * @param space
     * @param parentId
     * @param title
     * @param url
     * @param permissions
     * @param translations
     * @param version
     * @param minorVersion
     * @param content
     * @param created
     * @param creator
     * @param modified
     * @param modifier
     * @param homePage
     * @param contentStatus
     * @param current
     * @param language
     */
    public Page(String id, String space, String parentId, String title, String url,
        Integer permissions, List<String> translations, Integer version, Integer minorVersion,
        String content, Date created, String creator, Date modified, String modifier,
        Boolean homePage, String contentStatus, Boolean current, String language)
    {
        super(id, space, parentId, title, url, permissions, translations);
        this.version = version;
        this.content = content;
        this.created = created;
        this.creator = creator;
        this.modified = modified;
        this.modifier = modifier;
        this.homePage = homePage;
        this.contentStatus = contentStatus;
        this.current = current;
        this.language = language;

        checkFields();
    }

    /**
     * @param map A map containing the associations between XmlRpcObject field names and their
     *            corresponding values.
     * @see XmlRpcObject#XmlRpcObject(Map)
     */
    public Page(Map<String, Object> map)
    {
        super(map);
        checkFields();
    }

    /**
     * Confluence page constructor. Use this when creating new pages for Confluence. This
     * constructor is there for compatibility with Confluence. This is necessary because of the way
     * Confluence accepts the sent data, and the different semantics assigned to the id field.
     * The page id will be Space.Title when used with XWiki.
     * 
     * @param space
     * @param title
     * @param content
     */
    public Page(String space, String title, String content)
    {
        super("", space, "", title, "", 0, null);

        id = title;
        parentId = null;
        this.content = content;
        checkFields();
    }

    /**
     * Initialize and check that all the fields have correct values.
     */
    private void checkFields()
    {
        if (version == null) {
            version = 0;
        }

        if (minorVersion == null) {
            minorVersion = 1;
        }

        if (permissions == null) {
            permissions = 0;
        }

        if (language == null) {
            language = DEFAULT_LANGUAGE;
        }
    }

    public Integer getVersion()
    {
        return version;
    }

    public String getContent()
    {
        return content;
    }

    public Date getCreated()
    {
        return created;
    }

    public String getCreator()
    {
        return creator;
    }

    public Date getModified()
    {
        return modified;
    }

    public String getModifier()
    {
        return modifier;
    }

    public Boolean getHomePage()
    {
        return homePage;
    }

    public String getCurrentStatus()
    {
        return contentStatus;
    }

    public Boolean getCurrent()
    {
        return current;
    }

    public void setContent(String content)
    {
        this.content = content;
    }

    public void setTitle(String title)
    {
        this.title = title;
    }

    public String getName()
    {
        return name;
    }

    public Integer getMinorVersion()
    {
        return minorVersion;
    }

    public String getContentStatus()
    {
        return contentStatus;
    }

    public String getLanguage()
    {
        return language;
    }

    public void setParentId(String parentId)
    {
        this.parentId = parentId;
    }
}
