/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc.model;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * A class representing page summaries. Modeled after
 * {@link http://confluence.atlassian.com/display/DOC/Remote+API+Specification#RemoteAPISpecification-PageSummary}
 * The category XWikiSpecific is used to mark fields that have been added with respect to the
 * previous specification.
 * 
 * @author fmancinelli
 * @category ConfluenceAPI
 */
public class PageSummary extends XmlRpcObject
{
    @XmlRpcObjectField
    protected String id;

    @XmlRpcObjectField
    protected String space;

    @XmlRpcObjectField
    protected String parentId;

    @XmlRpcObjectField
    protected String title;

    @XmlRpcObjectField
    protected String url;

    @XmlRpcObjectField
    protected Integer permissions;

    /**
     * @category XWikiSpecific
     */
    @XmlRpcObjectField
    protected List<String> translations;

    /**
     * This field is synthesized from the id attribute and it represents the "Page" part of the full
     * "Space.Page" page identifier.
     * 
     * @category XWikiSpecific
     */
    protected String name;

    /**
     * Default constructor for creating PageSummary objects with no translations. Client should not
     * need to use this.
     * 
     * @param id
     * @param space
     * @param parentId
     * @param title
     * @param url
     * @param permissions
     */
    public PageSummary(String id, String space, String parentId, String title, String url,
        Integer permissions)
    {
        this(id, space, parentId, title, url, permissions, null);
        checkFields();
    }

    /**
     * Default constructor for creating PageSummary objects. Client should not need to use this.
     * 
     * @param id
     * @param space
     * @param parentId
     * @param title
     * @param url
     * @param permissions
     * @param translations
     */
    public PageSummary(String id, String space, String parentId, String title, String url,
        Integer permissions, List<String> translations)
    {
        super();

        this.id = id;
        this.space = space;
        this.parentId = parentId;
        this.title = title;
        this.url = url;
        this.permissions = permissions;
        this.translations = translations;

        checkFields();
    }

    /**
     * @param map A map containing the associations between XmlRpcObject field names and their
     *            corresponding values.
     * @see XmlRpcObject#XmlRpcObject(Map)
     */
    public PageSummary(Map<String, Object> map)
    {
        super(map);
        checkFields();
    }

    /**
     * Initialize and check that all the fields have correct values.
     */
    private void checkFields()
    {
        if (id == null && title == null) {
            throw new IllegalArgumentException("Page id or title must not be null");
        }
        else {
            if(id == null) {
                id = title;
            }
        }

        if (space == null) {
            throw new IllegalArgumentException("Space must not be null");
        }

        if (id.indexOf('.') != -1) {
            this.space = id.split("\\.")[0];
        }

        if (id.indexOf('.') != -1) {
            name = id.split("\\.")[1];
        } else {
            name = id;
            id = String.format("%s.%s", space, id);            
        }

        if (title == null) {
            title = "No title";
        }

        if (url == null) {
            url = "";
        }

        if (permissions == null) {
            permissions = 0;
        }

        if (translations == null) {
            translations = new ArrayList<String>();
        }
    }

    public String getId()
    {
        return id;
    }

    public String getSpace()
    {
        return space;
    }

    public String getParentId()
    {
        return parentId;
    }

    public String getTitle()
    {
        return title;
    }

    public String getUrl()
    {
        return url;
    }

    public Integer getPermissions()
    {
        return permissions;
    }

    public String getName()
    {
        return name;
    }

    public List<String> getTranslations()
    {
        return translations;
    }
}
