/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc.model;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Utility methods.
 * 
 * @author fmancinelli
 */
public class Utils
{
    /**
     * Date formats used for trying to convert strings to Date objects.
     */
    private static final SimpleDateFormat[] dateFormats =
        new SimpleDateFormat[] {
        new SimpleDateFormat("EEE MMM d HH:mm:ss z yyyy"),
        new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z"),
        new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.S"),
        new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SZ"),
        new SimpleDateFormat("yyyy-MM-dd HH:mm:ssZ"),
        new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"),
        new SimpleDateFormat("yyyy-MM-dd HH:mmZ"),
        new SimpleDateFormat("yyyy-MM-dd HH:mm"),
        // XML-RPC spec compliant iso8601 formats
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SZ"),
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S"),
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ"),
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss"),
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mmZ"), new SimpleDateFormat("yyyy-MM-dd'T'HH:mm"),
        new SimpleDateFormat("yyyy-MM-dd"), new SimpleDateFormat("yyyyMMdd")};

    /**
     * Convert a value to a suitable form for being transfered using XML-RPC. Arrays, Lists and Maps
     * are recursively converted as well. All other objects are implicitly converted to String by
     * using the toString() method. null values are encoded as strings and their actual encoding is
     * the one specified in the XmlRpcConstants.NULL_VALUE constant.
     * 
     * @param object The object to convert.
     * @return The converted object.
     */
    @SuppressWarnings("unchecked")
    public static Object convertToXmlRpc(Object object)
    {
        /*
         * Defensive programming and security check. If we have a null-object we convert to a
         * constant representing a null value. Otherwise XMLRPC would raise an exception. This
         * should not happen when serializing XmlRpcObjects beacuse the toMap method includes in the
         * serialization only non-null values.
         */
        if (object == null) {
            return XmlRpcConstants.NULL_VALUE;
        }

        if (object.getClass().isArray()) {
            Object[] objects = (Object[]) object;
            List result = new ArrayList();
            for (Object element : objects) {
                result.add(convertToXmlRpc(element));
            }

            return result;
        } else if (object instanceof List) {
            List list = (List) object;
            List result = new ArrayList();
            for (Object element : list) {
                result.add(convertToXmlRpc(element));
            }

            return result;
        } else if (object instanceof Map) {
            Map map = (Map) object;
            Map result = new HashMap();
            for (Object key : map.keySet()) {
                Object value = map.get(key);
                result.put(key, convertToXmlRpc(value));
            }

            return result;
        }

        /*
         * This would have been a cleaner way of handling Date objects, but doing so will break
         * compatibility with older versions on XWiki that expects data passed as String. If we
         * directly pass a Date object, in fact, we will incur in a class cast exception.
         */
        // else if (object instanceof Date) {
        // return object;
        // }
        /*
         * We could have also returned Integer and Double as is, but this breaks Confluence.
         * Moreover XWiki has a more rich data type model for objects, that comprises floats and
         * longs, which are not part of the XMLRPC data types. So if we have to send a long, we must
         * encode it as an Integer and on the server side there will be no way of saying that this
         * was supposed to be a long (modulo tedious XWiki class checking etc.). Since the
         * serialization number->string is very simple and since XWiki has set methods that directly
         * accepts String, we don't bother to send actual data types but we serialize everything
         * else as a string.
         */

        return object.toString();
    }

    /**
     * Convert a String to a given type. Supported types are Boolean, Integer, Double and Date.
     * 
     * @param string The string to be converted.
     * @param type The Class of the target type.
     * @return An object of the given type that represents the string to be converted.
     */
    @SuppressWarnings("unchecked")
    public static Object convertFromString(String string, Class type)
    {
        if (type.equals(String.class)) {
            return string;
        } else if (type.equals(Boolean.class)) {
            return new Boolean(string);
        } else if (type.equals(Integer.class)) {
            return new Integer(string);
        } else if (type.equals(Double.class)) {
            return new Double(string);
        } else if (type.equals(Date.class)) {
            for (SimpleDateFormat dateFormat : dateFormats) {
                try {
                    return dateFormat.parse(string);
                } catch (ParseException e) {
                    // Ignore
                }
            }
        }

        throw new IllegalArgumentException(String.format(
            "Cannot convert string '%s' to type %s\n", string, type.getName()));
    }

    /**
     * Truncate a string to the first '\n' character.
     * 
     * @param string
     * @return The truncated string.
     */
    public static String truncateToFirstLine(String string)
    {
        int newLineIndex = string.indexOf('\n');

        if (newLineIndex == -1) {
            return string;
        }

        return String.format("%s...", string.substring(0, newLineIndex));
    }
}
