/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * A class representing XWiki objects. An XWiki object contains the association between the
 * properties defined in the object's XWikiClass and their corresponding values.
 * 
 * @author fmancinelli
 * @category XWikiSpecific
 */
public class XWikiObject extends XWikiObjectSummary
{
    @XmlRpcObjectField
    protected Map<String, Object> propertyToValueMap;

    /**
     * Constructor for creating a new XWikiObject.
     * 
     * @param pageId
     * @param className
     * @param prettyName
     */
    public XWikiObject(String pageId, String className, String prettyName)
    {
        super(pageId, className, -1, prettyName);
        propertyToValueMap = new HashMap<String, Object>();
        checkFields();
    }

    /**
     * Default constructor for creating XWikiClassSummary objects. Client should not need to use
     * this.
     * 
     * @param pageId
     * @param className
     * @param id
     * @param prettyName
     * @param propertyToValueMap
     */
    public XWikiObject(String pageId, String className, Integer id, String prettyName,
        Map<String, Object> propertyToValueMap)
    {
        super(pageId, className, id, prettyName);
        this.propertyToValueMap = propertyToValueMap;
        checkFields();
    }

    /**
     * @param map A map containing the associations between XmlRpcObject field names and their
     *            corresponding values.
     * @see XmlRpcObject#XmlRpcObject(Map)
     */
    public XWikiObject(Map<String, Object> map)
    {
        super(map);
        checkFields();
    }

    /**
     * Initialize and check that all the fields have correct values.
     */
    private void checkFields()
    {
        if (propertyToValueMap == null) {
            throw new IllegalArgumentException("propertyToValueMap cannot be null.");
        }
    }

    public Set<String> getProperties()
    {
        return propertyToValueMap.keySet();
    }

    public Object get(String propertyName)
    {
        Object value = propertyToValueMap.get(propertyName);
        if (value != null) {
            /* Convert arrays to list for easier management */
            if (value.getClass().isArray()) {
                ArrayList array = new ArrayList();
                for (Object o : (Object[]) value) {
                    array.add(o);
                }
                value = array;
                propertyToValueMap.put(propertyName, value);
            }
        }

        return value;
    }

    public void set(String propertyName, Object value)
    {
        propertyToValueMap.put(propertyName, value);
    }
}
