/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.xmlrpc.model;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Formatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * XmlRpcObjects are used to encapsulate data to be transported using XML-RPC. Instance fields of an
 * XmlRpcObject are converted to a map('field name' -> 'value') that is used to send the
 * representation of the object through XMLRPC. Since there could be some fields that should not
 * appear in the representation (e.g., fields used as a cache for synthesized values), only the
 * fields annotated with the XmlRpcObjectField annotation are included in the map conversion. In
 * order to be correctly transported, field types must be one of the following: String, Boolean,
 * Integer, Double, Date. At instantiation, a check is performed on XmlRpcObjectField annotated
 * fields in order to prevent the instantiation of non well-formed objects. Arrays, Lists and Maps
 * of the previous elements are also supported (see {@link Utils#convertToXmlRpc(Object)}). In
 * order to create an XmlRpcObject you need to instantiate a class that derives from XmlRpcObject
 * and override the XmlRpcObject(Map<String, Object>) constructor, calling the super(Map<String,
 * Object>) constructor. You may need to perform some additional check in this constructor in order
 * to control that all the variables have consistent values. In fact, if a field name is not in the
 * provided map, its corresponding value will not be initialized (i.e., it will default to null),
 * and this might not be what you wanted. You can also provide ad ad-hoc constructor. Don't forget
 * to call the super() constructor to perform the sanity check. When an XmlRpcObject is initialized
 * through the XmlRpcObject(Map<String, Object>) constructor, all the fields associated to a String
 * value are automatically converted to their actual type. So it is safe to initialize Boolean,
 * Integers, and Double fields using strings.
 * 
 * @author fmancinelli
 */
public class XmlRpcObject
{

    /**
     * The default constructor performs a sanity check on the fields declared in a subclass of
     * XmlRpcObject. Allowed field types are: Boolean, Integer, Double, Date, Map, and List.
     */
    protected XmlRpcObject()
    {
        List<Field> xmlRpcObjectFields = getXmlRpcObjectFields();

        for (Field field : xmlRpcObjectFields) {
            if (!field.getType().equals(String.class) && !field.getType().equals(Boolean.class)
                && !field.getType().equals(Integer.class)
                && !field.getType().equals(Double.class) && !field.getType().equals(Date.class)
                && !field.getType().equals(Map.class) && !field.getType().equals(List.class)) {
                throw new RuntimeException(String.format(
                    "Field '%s.%s' has an invalid type '%s'\n", this.getClass().getName(), field
                        .getName(), field.getType()));
            }
        }
    }

    /**
     * Initialize the XmlRpcObject by providing a map containing the associations 'field name' ->
     * 'value'.
     * 
     * @param fieldToValueMap The map containing the associations 'field name' -> 'value'.
     */
    protected XmlRpcObject(Map<String, Object> fieldToValueMap)
    {
        this();

        List<Field> xmlRpcObjectFields = getXmlRpcObjectFields();

        for (Field field : xmlRpcObjectFields) {
            Object value = fieldToValueMap.get(field.getName());
            if (value != null) {
                if (value instanceof String) {
                    value = Utils.convertFromString((String) value, field.getType());
                }

                try {
                    /*
                     * XML-RPC supports lists but deserializes them as arrays. So if we get an array
                     * here we convert it back to a list. This means that an XmlRpcObjectField must
                     * not be declared as an array but as a list!
                     */
                    if (value.getClass().isArray()) {
                        value = Arrays.asList(((Object[]) value));
                    }

                    field.set(this, value);
                } catch (IllegalArgumentException e) {
                    throw e;
                } catch (IllegalAccessException e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }

    /**
     * Build a representation of the XmlRpcObject by using a map that associates to every field name
     * its corresponding value. Only the fields annotated with the XmlRpcObjectField annotation will
     * be added to the representation. Types are preserved.
     * 
     * @return A map containing the associations 'field name' -> 'value'.
     */
    public Map<String, Object> toMap()
    {
        Map<String, Object> result = new HashMap<String, Object>();

        List<Field> xmlRpcObjectFields = getXmlRpcObjectFields();

        for (Field field : xmlRpcObjectFields) {
            try {
                /* Serialize only non-null fields */
                if (field.get(this) != null) {
                    result.put(field.getName(), Utils.convertToXmlRpc(field.get(this)));
                }
            } catch (IllegalArgumentException e) {
                throw e;
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }

        return result;
    }

    /**
     * @return A list of all the fields annotated with the XmlRpcObjectField annotations that are
     *         declared in the object and in all its super-classes.
     */
    private List<Field> getXmlRpcObjectFields()
    {
        List<Field> result = new ArrayList<Field>();

        /**
         * Usually 'this' is a subclass of XmlRpcObject. We need to go up through the inheritance
         * hierarchy in order to find the fields declared in all the super classes. This is
         * necessary because reflection mechanisms are do not list the inherited fields.
         */
        Class currentClass = this.getClass();
        while (!currentClass.equals(XmlRpcObject.class)) {
            for (Field field : currentClass.getDeclaredFields()) {
                /* Only consider fields that are annotated with the XmlRpcObjectField annotation. */
                if (field.getAnnotation(XmlRpcObjectField.class) != null) {
                    result.add(field);
                }
            }

            currentClass = currentClass.getSuperclass();
        }

        return result;
    }

    @Override
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        Formatter f = new Formatter(sb);

        List<Field> fields = getXmlRpcObjectFields();

        f.format("%s[\n", this.getClass().getName());
        for (Field field : fields) {
            try {
                Object value = field.get(this);
                String stringValue = value != null ? value.toString() : "(null)";
                Object valueClass =
                    value != null ? value.getClass() : "No class info because value is null";
                f.format("  %s = '%s' (%s)\n", field.getName(), Utils
                    .truncateToFirstLine(stringValue), valueClass);
            } catch (Exception e) {
                f.format("  %s = Cannot access field (%s)\n", field.getName(), e);
            }
        }

        f.format("]\n");

        return sb.toString();
    }
}
